// Copyright 2016 The go-qemu Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package main

import (
	"fmt"
	"log"

	"github.com/digitalocean/go-qemu/hypervisor"
	"github.com/digitalocean/go-qemu/qemu"
)

// get the list of domains from libvirt
func pollHypervisor(hv *hypervisor.Hypervisor) string {
	var out string
	// fmt.Printf("\n**********Domains**********\n")
	domains, err := hv.Domains()
	if err != nil {
		log.Fatalf("Unable to get domains from hypervisor: %v", err)
	}
	for _, dom := range domains {
		var name string
		var err error
		var drives []string
		var status qemu.Status
		name = dom.Name
		// fmt.Printf("%s\n", dom.Name)
		status, err = dom.Status()
		if err != nil {
			fmt.Println(name, status, err)
			continue
		}
		// fmt.Printf("\nStatus: %s\n", status)
		/*
			if status == qemu.StatusRunning {
				// getBlockDevices(dom)
				// fmt.Println(name, status, err)
			}
		*/
		blockDevices, err := dom.BlockDevices()
		for _, blockDevice := range blockDevices {
			drives = append(drives, blockDevice.Device)
			drives = append(drives, blockDevice.Inserted.Driver)
			drives = append(drives, blockDevice.Inserted.File)
			// fmt.Printf("%20s %8s %30s\n",
			// 	blockDevice.Device, blockDevice.Inserted.Driver, blockDevice.Inserted.File)
		}
		if status == qemu.StatusRunning {
			fmt.Println("ON ", name, drives)
			out += "ON " + name + "\n"
		} else {
			fmt.Println("OFF", status, name, drives)
			out += "OFF " + name + "\n"
		}

		// Close the domain to clean up its resources and underlying monitor.
		// todo: store dom and reuse it? not closing it hangs libvirt eventually because it leaves dangling
		// connects that rack up and all block in virNetSocketReadWire:1781
		// probably this doesn't matter because NOTHING HERE IS DONE IN HIGH FREQUENCY EVER.
		// domU changes hardly ever happen
		if err := dom.Close(); err != nil {
			log.Fatalf("failed to close domain: %v", err)
		}
	}
	return out
}
